from .sql_types import (
    VARCHAR,
    CHAR,
    BLOB,
    TINYBLOB,
    TINYTEXT,
    MEDIUMBLOB,
    MEDIUMTEXT,
    LONGBLOB,
    LONGTEXT,
    ENUM,
    TEXT,
    STR,
    BOOL,
    INT,
    TINYINT,
    SMALLINT,
    MEDIUMINT,
    BIGINT,
    DECIMAL,
    FLOAT,
    DOUBLE,
    DATE,
    DATETIME,
    TIMESTAMP,
    TIME,
    YEAR
    )


type_map = {
    'VARCHAR':VARCHAR,
    'CHAR':CHAR,
    'BLOB':BLOB,
    'TINYBLOB':TINYBLOB,
    'TINYTEXT':TINYTEXT,
    'MEDIUMBLOB':MEDIUMBLOB,
    'MEDIUMTEXT':MEDIUMTEXT,
    'LONGBLOB':LONGBLOB,
    'LONGTEXT':LONGTEXT,
    'ENUM':ENUM,
    'TEXT':TEXT,
    'STR':STR,
    'BOOL':BOOL,
    'INT':INT,
    'TINYINT':TINYINT,
    'SMALLINT':SMALLINT,
    'MEDIUMINT':MEDIUMINT,
    'BIGINT':BIGINT,
    'DECIMAL':DECIMAL,
    'FLOAT':FLOAT,
    'DOUBLE':DOUBLE,
    'DATE':DATE,
    'DATETIME':DATETIME,
    'TIMESTAMP':TIMESTAMP,
    'TIME':TIME,
    'YEAR':YEAR,
    }

__all__ = (
    'VARCHAR',
    'CHAR',
    'BLOB',
    'TINYBLOB',
    'TINYTEXT',
    'MEDIUMBLOB',
    'MEDIUMTEXT',
    'LONGBLOB',
    'LONGTEXT',
    'ENUM',
    'TEXT',
    'STR',
    'BOOL',
    'INT',
    'TINYINT',
    'SMALLINT',
    'MEDIUMINT',
    'BIGINT',
    'DECIMAL',
    'FLOAT',
    'DOUBLE',
    'DATE',
    'DATETIME',
    'TIMESTAMP',
    'TIME',
    'YEAR',
    )