



class InvalidDate(TypeError):
    """
    Value must be a valid datetime object
    """
    pass
class NonConfiguredDataType(TypeError):
    """
    DataType not yet configured for auto-conversion
    """
    pass